@@ -1,22 +1,12 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8 -
 #
-# Copyright (c) 2008-2009 Benoit Chesneau <benoitc@e-engura.com> 
-#
-# Permission to use, copy, modify, and distribute this software for any
-# purpose with or without fee is hereby granted, provided that the above
-# copyright notice and this permission notice appear in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
-# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+# This file is part of couchdbkit released under the MIT license. 
+# See the NOTICE for more information.
 
 """
 All exceptions used in couchdbkit.
 """
+from restkit.errors import ResourceError
 
 class InvalidAttachment(Exception):
     """ raised when an attachment is invalid """
@@ -52,3 +42,19 @@ class BulkSaveError(Exception):
 
 class ViewServerError(Exception):
     """ exception raised by view server"""
+
+class MacroError(Exception):
+    """ exception raised when macro parsiing error in functions """
+
+class DesignerError(Exception):
+    """ unkown exception raised by the designer """
+
+class ResourceNotFound(ResourceError):
+    """ Exception raised when resource is not found"""
+
+class ResourceConflict(ResourceError):
+    """ Exception raised when there is conflict while updating"""
+
+class PreconditionFailed(ResourceError):
+    """ Exception raised when 412 HTTP error is received in response
+    to a request """
