@@ -13,13 +13,11 @@ ResourceConflict
 
 from couchdbkit import *
 
-from restkit import SimplePool
-pool = SimplePool()
 
 class ClientServerTestCase(unittest.TestCase):
     def setUp(self):
         self.couchdb = CouchdbResource()
-        self.Server = Server(pool_instance=pool)
+        self.Server = Server()
         
     def tearDown(self):
         try:
@@ -57,6 +55,14 @@ class ClientServerTestCase(unittest.TestCase):
         self.assert_(db.dbname == gocdb.dbname)
         self.Server.delete_db("get_or_create_db")
         
+
+    def testCreateInvalidDbName(self):
+
+        def create_invalid():
+            res = self.Server.create_db('123ab')
+
+        self.assertRaises(ValueError, create_invalid) 
+    
     def testBadResourceClassType(self):
         self.assertRaises(TypeError, Server, resource_class=None)
         
@@ -378,6 +384,17 @@ class ClientDatabaseTestCase(unittest.TestCase):
         fetch_attachment = db.fetch_attachment(doc, "test")
         self.assert_(text_attachment == fetch_attachment)
         del self.Server['couchdbkit_test']
+        
+    def testFetchAttachmentStream(self):
+        db = self.Server.create_db('couchdbkit_test')
+        doc = { 'string': 'test', 'number': 4 }
+        db.save_doc(doc)        
+        text_attachment = u"a text attachment"
+        db.put_attachment(doc, text_attachment, "test", "text/plain")
+        stream = db.fetch_attachment(doc, "test", stream=True)
+        fetch_attachment = stream.read()
+        self.assert_(text_attachment == fetch_attachment)
+        del self.Server['couchdbkit_test']
    
     def testEmptyAttachment(self):
         db = self.Server.create_db('couchdbkit_test')
