@@ -1,21 +1,11 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8 -
 #
-# Copyright (c) 2009 Benoit Chesneau <benoitc@e-engura.com> 
-#
-# Permission to use, copy, modify, and distribute this software for any
-# purpose with or without fee is hereby granted, provided that the above
-# copyright notice and this permission notice appear in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
-# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+# This file is part of couchdbkit released under the MIT license. 
+# See the NOTICE for more information.
 #
 __author__ = 'benoitc@e-engura.com (Benoît Chesneau)'
 
+import base64
 import os
 import shutil
 import tempfile
@@ -38,6 +28,8 @@ class LoaderTestCase(unittest.TestCase):
         self.template_dir = os.path.join(os.path.dirname(__file__), 'data/app-template')
         self.app_dir = os.path.join(self.tempdir, "couchdbkit-test")
         shutil.copytree(self.template_dir, self.app_dir)
+        write_content(os.path.join(self.app_dir, "_id"),
+                "_design/couchdbkit-test")
         self.server = Server()
         self.db = self.server.create_db('couchdbkit_test')
         
@@ -88,8 +80,8 @@ class LoaderTestCase(unittest.TestCase):
         self.assert_('index.html' in design_doc['_attachments'])
         self.assert_('style/main.css' in design_doc['_attachments'])
         
-        content = open(design_doc['_attachments']['style/main.css'], 'rb').read()
-        self.assert_(content == "/* add styles here */")
+        content = design_doc['_attachments']['style/main.css']
+        self.assert_(base64.b64decode(content['data']) == "/* add styles here */")
         
     def testGetDocSignatures(self):
         l = FileSystemDocsLoader(self.tempdir)
